# coding=utf-8

import hashlib
from urllib.parse import urlparse, parse_qs, parse_qsl,urlencode
import string
import requests

class GenSignData():

    def __init__(self):
        self.carappkey = "@7U$aPOE@$"
        sign = ""
        # a = {"_appid":2, _timestamp=sdf, ask=3, cid=2, cname=d, conid=e, deviceid=2, isfirst=2, latitude=df, longitude=df, osver=sdf, pid=2, pm=1, pname=df, projectid=33, refer=d, source=23, userid=2, version=3}
        # sorted()

    # def get_sign_url(self, url):
    #     """
    #     {_appid=app.iphone, _timestamp=1531995896, ask=全景天窗。, cid=110100, cname=北京, conid=, deviceid=f7afe07726d41444b6c449175e14d018e90db0c2, isfirst=0, latitude=39.98532785602838, longitude=116.3187323094177, osver=0, pid=110000, pm=1, pname=北京, projectid=2, refer=, source=0, userid=0, version=9.3.5}
    #     :param url:
    #     :return: _sign
    #     """
    #     u = urlparse(url)
    #     q = parse_qsl(u.query, keep_blank_values=True)
    #     sort_q = sorted(q)
    #     join_q = ""
    #     for x, y in sort_q:
    #         if x == "_sign":
    #             pass
    #         else:
    #             join_q = join_q + x + y
    #     join_q = self.carappkey + join_q + self.carappkey
    #     _sign = self._encrypt_sign(join_q)
    #     new_q = parse_qs(u.query, keep_blank_values=True)
    #     new_q["_sign"] = [_sign]
    #     new_join_q = urlencode(new_q, doseq=True)
    #     print("new_jon_q", new_join_q)
    #     end_u = u._replace(query=new_join_q)
    #     return _sign, end_u.geturl()

    def get_params_sign(self, q):
        sort_q = sorted(q.items())
        join_q = ""
        for x, y in sort_q:
            if x == "_sign":
                pass
            else:
                join_q = join_q + str(x) + str(y)
        join_q = self.carappkey + join_q + self.carappkey
        m2 = hashlib.md5()
        m2.update(join_q.encode("utf-8"))
        sign_d = str(m2.hexdigest())
        _sign = sign_d.upper()
        q["_sign"] = _sign
        return q

    # def _get_sign_query(self, query):
    #     join_q = ""
    #     for x, y in query.items():
    #         join_q = join_q + str(x) + str(y)
    #     join_q = self.carappkey + join_q + self.carappkey
    #     return self._encrypt_sign(join_q)


    # def _encrypt_sign(self, querys):
    #     m2 = hashlib.md5()
    #     m2.update(querys.encode("utf-8"))
    #     sign_d = str(m2.hexdigest())
    #     return sign_d.upper()



if __name__ == '__main__':
    g = GenSignData()
    # url = "https://newsbj.app.autohome.com.cn/carhelper_v9.3.0/news/carhelper.ashx?userid=0&pm=2&source=3&thinktype=&ask=%E5%AE%9D%E6%9D%A5&pname=%E5%8C%97%E4%BA%AC&osver=0&cname=%E5%8C%97%E4%BA%AC&_timestamp=1532416054&latitude=39.98533139448861&projectid=2&conid=S4EVqiw0ss84oVtGm&version=9.5.0&refer=&cid=110100&deviceid=f7afe07726d41444b6c449175e14d018e90db0c2&isfirst=0&longitude=116.3187476653883&pid=110000&_appid=app"
    # _, new_url = g.get_sign_url(url)
    # print(_)
    # old_res = requests.get(url)
    # print("old_url:" ,old_res.json())
    # new_res = requests.get(new_url)
    # print("new_url:", new_res.json())
    # print(new_url)
    query = {'userid': '0', 'pm': '2', 'source': '3', 'thinktype': '', 'ask': '宝来', 'pname': '北京', 'osver': '0', 'cname': '北京', '_timestamp': '1532416054', 'latitude': '39.98533139448861', 'projectid': '2', 'conid': 'S4EVqiw0ss84oVtGm', 'version': '9.5.0', 'refer': '', 'cid': '110100', 'deviceid': 'f7afe07726d41444b6c449175e14d018e90db0c2', 'isfirst': '0', 'longitude': '116.3187476653883', 'pid': '110000', '_appid': 'app'}
    m = g.get_params_sign(query)
    r = requests.get(url="https://newsbj.app.autohome.com.cn/carhelper_v9.3.0/news/carhelper.ashx",
                 params=m)
    print(m)
    print(r.json())






import threading
import subprocess
import time


# def run_comm(comm):
#     """执行comm"""
#     try:
#         p = subprocess.Popen(
#             comm,
#             stdin=subprocess.PIPE,
#             stdout=subprocess.PIPE,
#             stderr=subprocess.PIPE,
#             shell=True,
#             # start_new_session=True
#         )
#         return p
#     except Exception as e:
#         print("执行失败：%s\n 命令：%s" % (e, comm))
#
#
# def haha(m):
#     comm = "C:\\Users\\wh\\.venv\\AutoCreateScript\\Scripts\\python.exe test_image.py %s" % m
#     p = run_comm(comm)
#     print("*" * 10, "start", "*" * 10)
#     for x in p.stdout.readlines():
#         print(x)
#     for y in p.stderr.readlines():
#         print(y)
#     return
#
# def one_by_one():
#     for x in range(20):
#         haha("abc")
#
# def haha_run():
#     startTime = time.time()
#     print("start:", time.ctime())
#     one_by_one()
#     endTime = time.time()
#     print("end:  ", time.ctime())
#     timeToken = "%.3f" %(endTime - startTime)
#     print(timeToken)
#
# # haha_run()
# #
# startTime = time.time()
# print("start:", time.ctime())
# for x in range(20):
#     t = threading.Thread(target=haha,args=("abc",))
#     t.start()
#     # t.join()
# print("current has %d threads" % (threading.activeCount() - 1))
# endTime = time.time()
# print("end:  ", time.ctime())
# timeToken = "%.3f" %(endTime - startTime)
# print(timeToken)

import requests
callback_url = 'http://192.168.225.225/testsystem/caseinfo/backinfo/19173/3893/13365.do'
# report  = {'interface_name': '这里是类描述', 'case_name': 'TestImage', 'case_url': '获取CASE失败', 'req_time': '2018-12-28 10:44:43', 'time_token': '208.452', 'http_status': None, 'head_ip': 'fe80::4087:f766:405e:28e6%7', 'result_status': '请求异常', 'content': None}
# res = requests.post(url=callback_url, data=report)
# print(res.status_code)
url = "http://www.baidu.com"
res = requests.get(url)
print(res.headers.get("Content-Type") == "application/json")





class GenerateScript:
    """
    类名以接口方法名进行生成，最前面加Test，将首字母大写，其余小写
    如方法名为：/services/dealerFactorySerise/get
    文件名为：test_services_dealerfactoryserise_get.py
    测试类名为：TestServicesDealerfactorySeriesGet
    """

    def __init__(self, scriptname, classname, description):
        self.scriptname = scriptname
        self.classname = classname
        self.description = description

    def generate_script(self):
        """
        生成测试脚本方法
        :return:
        """
        # script_name = "test" + "_".join([name.lower() for name in self.path.split("/")]) + ".py"
        # class_name = "Test" + "".join([name.title() for name in self.path.split("/")])
        # class_description = "这里是类描述"
        script_data = """# coding=utf-8
import unittest
import requests
from base import *

class %(classname)s(InterFaceBaseTest):
    \"\"\"
    %(class_description)s
    \"\"\"

    @classmethod
    def setUpClass(cls, ids=%(ids)d):
        \"\"\"
        向超类传参，如无必要请勿修改
        :param ids: 获取数据的id
        :return:
        \"\"\"
        transfer = {
            "ids": sys.argv[1],
            "env": sys.argv[2]
        }

        super(%(classname)s, cls).setUpClass(ids)
        # do your something
        
    # def setUp(self):
    #     \"\"\"
    #     覆写超类setup
    #     :return:
    #     \"\"\"
    #     super(TestImage, self).setUp()
    #     # do your something

    # ============testcase begin==============
    
    # # write yourself testcase
    # def test_yourself_interface_testcase(self):
    #     \"\"\"
    #     你的接口测试用例描述
    #     :return: 
    #     \"\"\"
    #     pass
    
    
    # ============testcase end==============

    # def tearDown(self):
    #     \"\"\"
    #     覆写超类teardown
    #     :return:
    #     \"\"\"
    #     super(TestImage, self).tearDown()
    #     # do your something

    # @classmethod
    # def tearDownClass(cls):
    #     \"\"\"
    #     向超类传参，如无必要请勿修改
    #     :return:
    #     \"\"\"
    #     # do your something
    #     super(TestImage, cls).tearDownClass()

with BaseRunSuite(%(classname)s): pass

""" % dict(classname=self.classname, class_description=self.description, ids=12)

        print("script_name is : %s " % self.scriptname)
        print("class name is : %s " % self.classname)
        print(script_data)

        with open(self.scriptname, "w", encoding="utf-8") as fp:
            fp.writelines(script_data)


if __name__ == '__main__':
    g = GenerateScript("test_script_name.py", "Test_name", "this is autocreate")
    g.generate_script()



# coding=utf-8

import requests
import unittest
import time
import sys
import hashlib
from urllib.parse import urlparse
import socket
import logging
import json

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s'
)


class AnalysisData(object):
    """
    analysis json module
    """

    def __init__(self, ids, env):
        self.data_url = "http://192.168.225.225/testsystem/caseinfo/testjob?id=%s&env=%s" % (ids, env)
        # self.data_url = "http://192.168.225.224/break/jsonmodule/"
        self.session = requests.Session()
        a = requests.adapters.HTTPAdapter(max_retries=3)
        self.session.mount('http://', a)
        self.session.mount('https://', a)
        self.method = None
        self.data = None
        self.params = None
        self.json = None
        self.timeout = 5
        self.header = None
        self.url = None
        self.posttype = None
        self.verify = None
        self.response_status = 0

    def __enter__(self):
        try:
            res = requests.get(self.data_url)
            self.res_json = res.json()
            self._set_url()
            self._set_params()
            self._add_host_ip()
            self._get_response_status()
            return self.req_session(), self.res_json, self._get_response_key_and_value()
        except Exception as e:
            return None, self.res_json, None

    def _set_params(self):
        self.method = self._get_method()
        if self.method == "GET":
            self.params = self._get_params()
        elif self.method == "POST":
            self.posttype = self._get_posttype()
            if self.posttype == "JSON":
                self.json = self._get_params()
            else:
                self.data = self._get_params()

    def _get_method(self):
        """
        get req method
        :return: method get post
        """
        method = self.res_json.get("requestdata").get("method")
        method = str.upper(method)
        return method

    def _get_posttype(self):
        posttype = self.res_json.get("requestdata").get("posttype")
        posttype = str.upper(posttype)
        return posttype

    def _set_url(self):
        url = self.res_json.get("requestdata").get("url")
        self.url = url

    def _get_params(self):
        """
        get params if encryption then encryp
        :return:
        """
        encryption = self.res_json.get("requestdata").get("encryption")
        params = self.res_json.get("requestdata").get("param")
        if encryption:
            for ev in encryption:
                if isinstance(ev, dict) and ev:
                    if str.upper(ev.get("type") if ev.get("type") else "") == "SIGN":
                        appkey = ev.get("appkey")
                        params_add_sign = GenSignData(appkey).get_params_sign(params)
                        return params_add_sign
                    elif str.upper(ev.get("type") if ev.get("type") else "") == "AUTH":
                        # todo encryption
                        en_query = ""
                        return en_query
                    else:
                        return params
                else:
                    return params
        else:
            return params

    def _add_host_ip(self):
        req_head_ip = self.res_json.get("requestdata").get("ip")
        if req_head_ip:
            u = urlparse(self.url)
            domain = u.netloc
            u = u._replace(netloc=req_head_ip)
            self.header = {
                "Host": domain
            }
            self.url = u.geturl()
            self.verify = False

    def req_session(self):
        """
        get a request session, used to get response stuff
        :return: a request session
        """
        self.session = self.session.request(
            method=self.method,
            headers=self.header,
            params=self.params,
            data=self.data,
            json=self.json,
            url=self.url,
            timeout=self.timeout,
            verify=self.verify
        )
        return self.session

    def _get_response_status(self):
        response_status = self.res_json.get("returncheck").get("status")
        self.response_status = response_status

    def _get_response_key_and_value(self):
        backparams = self.res_json.get("returncheck").get("backparam")
        return backparams

    def __exit__(self, exc_type, exc_val, exc_tb):
        # pass
        self.session.close()
        # return None, self.res_json, None


class InterFaceBaseTest(unittest.TestCase):
    """
    base test class
    """

    @classmethod
    def setUpClass(cls, ids, env):
        """
        get parameter and create request session
        :return:
        """
        if len(env) < 2:
            print("missing positional argument")
            sys.exit(1)
        try:
            with AnalysisData(ids, env) as Ad:
                cls.session, cls.res_json, cls.backparams = Ad
                if "application/json" in str.lower(cls.session.headers.get("Content-Type")):
                    cls.res_module = GenResponseModule().get_json_result_keys_and_value_type(cls.session.json())
        except requests.exceptions.ConnectionError as e:
            print(e)
            assert False

    # def setUp(self):
    #     pass

    def test_http_code(self):
        """
        test http code
        :return:
        """
        self.assertEqual(
            self.session.status_code,
            200,
            "HTTP CODE不为200，判定失败！"
        )

    def test_req_elapsed(self):
        """
        test request time spend
        :return:
        """
        self.assertLessEqual(
            self.session.elapsed.total_seconds(),
            1,
            "请求时长超过1000ms，请关注，如频繁出现，建议做压力测试！"
        )

    def test_response_module(self):
        """
        test the response & module type is match
        验证规则：
        假设：A为平台接口返回的json module， B为用例response解析出的json module
        若A中节点must为true，则需要验证B中节点的path、value、type的值是否与A中相等，相等则通过，不等则未通过；
        若A节点must为false，则如果A中节点path匹配到B中节点path，则验证path对应的type是否相等，相等则通过，不等则不通过；
        如果A中节点path未匹配到B中path，则不验证；
        https://stackoverflow.com/questions/19031953/skip-unittest-test-without-decorator-syntax
        :return:
        """
        htype = self.session.headers.get("Content-Type")
        htype = str.lower(htype)
        if "application/json" not in htype:
            raise unittest.case.SkipTest("返回格式不为JSON，跳过！")
        for x in self.backparams:
            if x.get("must"):
                # 验证key存在
                self.assertIn(
                    x.get("path"),
                    self.res_module.keys(),
                    "字段must为True，key值：%s 不存在！" % x.get("path")
                )
                # 验证key类型正确
                if x.get("type") is not None:
                    self.assertEqual(
                        x.get("type"),
                        self.res_module.get(x.get("path")).get("type"),
                        "字段must为True，type类型：%s 不正确！" % str(x.get("type"))
                    )
                # 如果有值，则值要一致
                if x.get("value"):
                    self.assertEqual(
                        x.get("value"),
                        self.res_module.get(x.get("path")).get("value"),
                        "字段must为True，有value的情况下，value值%s不正确" % str(x.get("value"))
                    )
            else:
                # 如果有对应的key，则key对应的类型要一致
                if x.get("path") in self.res_module.keys() and x.get("type") is not None:
                    self.assertEqual(
                        x.get("type"),
                        self.res_module.get(x.get("path")).get("type"),
                        "字段must为False，key值对应类型%s与原类型%s不一致。" % (
                            str(x.get("type")),
                            str(self.res_module.get(x.get("path")).get("type"))
                        )
                    )

    # def tearDown(self):
    #     pass

    # @classmethod
    # def tearDownClass(cls):
    #     """
    #     recycing request session and clean everything
    #     :return:
    #     """
    #     pass


class BaseRunSuite(object):
    """
    suite testcase base class
    """

    def __init__(self, case):
        self.case = case

    def __enter__(self):
        self.result, self.start_time, self.timeToken = self.run()

    def run(self):
        suite = unittest.TestSuite()
        suite.addTest(unittest.TestLoader().loadTestsFromTestCase(self.case))
        runner = unittest.TextTestRunner(verbosity=2)
        startTime = time.time()
        result = runner.run(suite)
        stopTime = time.time()
        time_token = "%.3f" % (stopTime - startTime)
        return result, startTime, time_token

    def up_result(self):
        if self.result.testsRun == 0:
            result_status = "请求异常"
        elif self.result.wasSuccessful():
            result_status = "成功"
        elif len(self.result.failures):
            result_status = "用例失败"
        elif len(self.result.errors):
            result_status = "代码错误"
        else:
            result_status = "未知错误"
        interface_name = self.case.__doc__ and self.case.__doc__.split("\n")[1].strip() or ""
        case_name = self.case.__name__
        try:
            case_url = self.case.session.url
            logging.info("用例URL：%s" % case_url)
        except:
            logging.error("用例URL未获取到")
            case_url = None
        try:
            content = self.case.session.content
        except:
            logging.error("用例response未获取到")
            content = None
        try:
            http_status = self.case.session.status_code
            logging.info("请求状态码：%s" % http_status)
        except:
            logging.error("用例的状态码未获取到")
            http_status = None
        try:
            callback_url = self.case.res_json.get("callbackurl")
            logging.info("回调URL：%s" % callback_url)
        except:
            logging.error("回调URL未获取到")
            callback_url = None
        try:
            u = urlparse(case_url)
            domain_ip = socket.getaddrinfo(u.netloc, u.scheme)[0][4][0]
        except:
            domain_ip = None
        log = self.result.separator1 + "\n"
        for x in self.result.errors:
            log = log + x[1] + self.result.separator2 + "\n"
        for y in self.result.failures:
            log = log + y[1] + self.result.separator2 + "\n"
        log += self.result.separator1 + "\n"
        log += json.dumps(self.case.res_json, indent=4)
        report = {
            "interface_name": interface_name,
            "case_name": case_name,
            "case_url": case_url,
            "req_time": time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.start_time)),
            "time_token": self.timeToken,
            "http_status": http_status,
            "head_ip": domain_ip,
            "result_status": result_status,
            "content": content,
            "log": log
        }
        try:
            requests.post(callback_url, data=report, timeout=10)
            logging.info("回调成功")
        except Exception as e:
            logging.error("回调URL请求失败，原因: \n%s" % e)
        return report

    def __exit__(self, exc_type, exc_val, exc_tb):
        # callback data
        self.up_result()
        logging.info("timeToken: %s" % self.timeToken)


class GenResponseModule(object):
    """
    analysis response data module
    """

    def __init__(self):
        self.res_keys_value_type = dict()

    def get_json_result_keys_and_value_type(self, json_dict, key=""):
        """
        将json数据中的key按级别取出，字典级别用/连接，数组级别用[]连接，并排重
        并保存该key值对应的value的类型
        例如：{'code': 'int', 'message': 'str', 'result': 'dict', 'result.has_next': 'bool', 'result.list': 'list', 'result.list-apkname': 'str'}
        :param json_dict: 目标json数据，必须是dict类型
        :param key: 初始值，不用传
        :return: 返回要对比的排重后的数组
        """
        if isinstance(json_dict, list):
            for element in json_dict:
                self.get_json_result_keys_and_value_type(element, key + "[]")
        elif isinstance(json_dict, dict):
            for x in json_dict.keys():
                temp_key = key == "" and x or key + "/" + x
                value_type = not isinstance(json_dict[x], (list, dict)) and {
                    "type": self._change_type_name(json_dict[x]), "value": json_dict[x]} or {
                                 "type": self._change_type_name(json_dict[x])}
                self.res_keys_value_type[str(temp_key)] = value_type
                self.get_json_result_keys_and_value_type(json_dict[x], temp_key)
        return self.res_keys_value_type

    def _change_type_name(self, data):
        if isinstance(data, str):
            return "string"
        if isinstance(data, dict):
            return "object"
        if isinstance(data, list):
            return "array"
        if isinstance(data, bool):
            return "bool"
        if isinstance(data, int):
            return "int"
        if isinstance(data, float):
            return "float"


class GenSignData(object):
    """
    params encryp class
    """

    def __init__(self, appkey):
        # carhelper "@7U$aPOE@$"
        self.appkey = appkey

    def get_params_sign(self, q):
        """
        get _sign param
        :param q: not encryp params
        :return: add _sign params
        """
        if isinstance(q, dict):
            sort_q = sorted(q.items())
            join_q = ""
            for x, y in sort_q:
                if x == "_sign":
                    pass
                else:
                    join_q = join_q + str(x) + str(y)
            join_q = self.appkey + join_q + self.appkey
            m2 = hashlib.md5()
            m2.update(join_q.encode("utf-8"))
            sign_d = str(m2.hexdigest())
            _sign = sign_d.upper()
            q["_sign"] = _sign
            return q



import setuptools

with open("README.MD", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ibase",
    version="0.0.1",
    install_requires=[
        "requests"
    ],
    author="hanz",
    author_email="wanghan0406@autohome.com.cn",
    description="interface testcase script basefunc",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)